#version 330 core

in vec2 UV;
uniform sampler2D inputtex;
uniform sampler2D emitpos; // emit positions in emitpos.xy
uniform float init;
uniform float iGlobalTime;
uniform float scenenum;
uniform float deltat;
uniform float veloscale;
layout (location = 0) out vec4 color;
const int kTexSize = 128;
const float kTexScale = 1.0 / float(kTexSize);

// 3d perlin https://github.com/BrianSharpe/GPU-Noise-Lib/blob/master/gpu_noise_lib.glsl
vec4 FAST32_hash_2D_Cell( vec2 gridcell )   //  generates 4 different random numbers for the single given cell point
{
    //  gridcell is assumed to be an integer coordinate
    const vec2 OFFSET = vec2( 26.0, 161.0 );
    const float DOMAIN = 71.0;
    const vec4 SOMELARGEFLOATS = vec4( 951.135664, 642.949883, 803.202459, 986.973274 );
    vec2 P = gridcell - floor(gridcell * ( 1.0 / DOMAIN )) * DOMAIN;
    P += OFFSET.xy;
    P *= P;
    return fract( (P.x * P.y) * ( 1.0 / SOMELARGEFLOATS.xyzw ) );
}
vec4 FAST32_hash_3D_Cell( vec3 gridcell )   //  generates 4 different random numbers for the single given cell point
{
    //    gridcell is assumed to be an integer coordinate

    //  TODO:   these constants need tweaked to find the best possible noise.
    //          probably requires some kind of brute force computational searching or something....
    const vec2 OFFSET = vec2( 50.0, 161.0 );
    const float DOMAIN = 69.0;
    const vec4 SOMELARGEFLOATS = vec4( 635.298681, 682.357502, 668.926525, 588.255119 );
    const vec4 ZINC = vec4( 48.500388, 65.294118, 63.934599, 63.279683 );

    //  truncate the domain
    gridcell.xyz = gridcell - floor(gridcell * ( 1.0 / DOMAIN )) * DOMAIN;
    gridcell.xy += OFFSET.xy;
    gridcell.xy *= gridcell.xy;
    return fract( ( gridcell.x * gridcell.y ) * ( 1.0 / ( SOMELARGEFLOATS + gridcell.zzzz * ZINC ) ) );
}
void FAST32_hash_3D( vec3 gridcell, out vec4 lowz_hash, out vec4 highz_hash )   //  generates a random number for each of the 8 cell corners
{
    //    gridcell is assumed to be an integer coordinate

    //  TODO:   these constants need tweaked to find the best possible noise.
    //          probably requires some kind of brute force computational searching or something....
    const vec2 OFFSET = vec2( 50.0, 161.0 );
    const float DOMAIN = 69.0;
    const float SOMELARGEFLOAT = 635.298681;
    const float ZINC = 48.500388;

    //  truncate the domain
    gridcell.xyz = gridcell.xyz - floor(gridcell.xyz * ( 1.0 / DOMAIN )) * DOMAIN;
    vec3 gridcell_inc1 = step( gridcell, vec3( DOMAIN - 1.5 ) ) * ( gridcell + 1.0 );

    //  calculate the noise
    vec4 P = vec4( gridcell.xy, gridcell_inc1.xy ) + OFFSET.xyxy;
    P *= P;
    P = P.xzxz * P.yyww;
    highz_hash.xy = vec2( 1.0 / ( SOMELARGEFLOAT + vec2( gridcell.z, gridcell_inc1.z ) * ZINC ) );
    lowz_hash = fract( P * highz_hash.xxxx );
    highz_hash = fract( P * highz_hash.yyyy );
}
void FAST32_hash_3D(    vec3 gridcell,
                        vec3 v1_mask,       //  user definable v1 and v2.  ( 0's and 1's )
                        vec3 v2_mask,
                        out vec4 hash_0,
                        out vec4 hash_1,
                        out vec4 hash_2 )       //  generates 3 random numbers for each of the 4 3D cell corners.  cell corners:  v0=0,0,0  v3=1,1,1  the other two are user definable
{
    //    gridcell is assumed to be an integer coordinate

    //  TODO:   these constants need tweaked to find the best possible noise.
    //          probably requires some kind of brute force computational searching or something....
    const vec2 OFFSET = vec2( 50.0, 161.0 );
    const float DOMAIN = 69.0;
    const vec3 SOMELARGEFLOATS = vec3( 635.298681, 682.357502, 668.926525 );
    const vec3 ZINC = vec3( 48.500388, 65.294118, 63.934599 );

    //  truncate the domain
    gridcell.xyz = gridcell.xyz - floor(gridcell.xyz * ( 1.0 / DOMAIN )) * DOMAIN;
    vec3 gridcell_inc1 = step( gridcell, vec3( DOMAIN - 1.5 ) ) * ( gridcell + 1.0 );

    //  compute x*x*y*y for the 4 corners
    vec4 P = vec4( gridcell.xy, gridcell_inc1.xy ) + OFFSET.xyxy;
    P *= P;
    vec4 V1xy_V2xy = mix( P.xyxy, P.zwzw, vec4( v1_mask.xy, v2_mask.xy ) );     //  apply mask for v1 and v2
    P = vec4( P.x, V1xy_V2xy.xz, P.z ) * vec4( P.y, V1xy_V2xy.yw, P.w );

    //  get the lowz and highz mods
    vec3 lowz_mods = vec3( 1.0 / ( SOMELARGEFLOATS.xyz + gridcell.zzz * ZINC.xyz ) );
    vec3 highz_mods = vec3( 1.0 / ( SOMELARGEFLOATS.xyz + gridcell_inc1.zzz * ZINC.xyz ) );

    //  apply mask for v1 and v2 mod values
    v1_mask = ( v1_mask.z < 0.5 ) ? lowz_mods : highz_mods;
    v2_mask = ( v2_mask.z < 0.5 ) ? lowz_mods : highz_mods;

    //  compute the final hash
    hash_0 = fract( P * vec4( lowz_mods.x, v1_mask.x, v2_mask.x, highz_mods.x ) );
    hash_1 = fract( P * vec4( lowz_mods.y, v1_mask.y, v2_mask.y, highz_mods.y ) );
    hash_2 = fract( P * vec4( lowz_mods.z, v1_mask.z, v2_mask.z, highz_mods.z ) );
}
vec4 FAST32_hash_3D(    vec3 gridcell,
                        vec3 v1_mask,       //  user definable v1 and v2.  ( 0's and 1's )
                        vec3 v2_mask )      //  generates 1 random number for each of the 4 3D cell corners.  cell corners:  v0=0,0,0  v3=1,1,1  the other two are user definable
{
    //    gridcell is assumed to be an integer coordinate

    //  TODO:   these constants need tweaked to find the best possible noise.
    //          probably requires some kind of brute force computational searching or something....
    const vec2 OFFSET = vec2( 50.0, 161.0 );
    const float DOMAIN = 69.0;
    const float SOMELARGEFLOAT = 635.298681;
    const float ZINC = 48.500388;

    //  truncate the domain
    gridcell.xyz = gridcell.xyz - floor(gridcell.xyz * ( 1.0 / DOMAIN )) * DOMAIN;
    vec3 gridcell_inc1 = step( gridcell, vec3( DOMAIN - 1.5 ) ) * ( gridcell + 1.0 );

    //  compute x*x*y*y for the 4 corners
    vec4 P = vec4( gridcell.xy, gridcell_inc1.xy ) + OFFSET.xyxy;
    P *= P;
    vec4 V1xy_V2xy = mix( P.xyxy, P.zwzw, vec4( v1_mask.xy, v2_mask.xy ) );     //  apply mask for v1 and v2
    P = vec4( P.x, V1xy_V2xy.xz, P.z ) * vec4( P.y, V1xy_V2xy.yw, P.w );

    //  get the z mod vals
    vec2 V1z_V2z = vec2( v1_mask.z < 0.5 ? gridcell.z : gridcell_inc1.z, v2_mask.z < 0.5 ? gridcell.z : gridcell_inc1.z );
    vec4 mod_vals = vec4( 1.0 / ( SOMELARGEFLOAT + vec4( gridcell.z, V1z_V2z, gridcell_inc1.z ) * ZINC ) );

    //  compute the final hash
    return fract( P * mod_vals );
}
void FAST32_hash_3D(    vec3 gridcell,
                        out vec4 lowz_hash_0,
                        out vec4 lowz_hash_1,
                        out vec4 lowz_hash_2,
                        out vec4 highz_hash_0,
                        out vec4 highz_hash_1,
                        out vec4 highz_hash_2   )       //  generates 3 random numbers for each of the 8 cell corners
{
    //    gridcell is assumed to be an integer coordinate

    //  TODO:   these constants need tweaked to find the best possible noise.
    //          probably requires some kind of brute force computational searching or something....
    const vec2 OFFSET = vec2( 50.0, 161.0 );
    const float DOMAIN = 69.0;
    const vec3 SOMELARGEFLOATS = vec3( 635.298681, 682.357502, 668.926525 );
    const vec3 ZINC = vec3( 48.500388, 65.294118, 63.934599 );

    //  truncate the domain
    gridcell.xyz = gridcell.xyz - floor(gridcell.xyz * ( 1.0 / DOMAIN )) * DOMAIN;
    vec3 gridcell_inc1 = step( gridcell, vec3( DOMAIN - 1.5 ) ) * ( gridcell + 1.0 );

    //  calculate the noise
    vec4 P = vec4( gridcell.xy, gridcell_inc1.xy ) + OFFSET.xyxy;
    P *= P;
    P = P.xzxz * P.yyww;
    vec3 lowz_mod = vec3( 1.0 / ( SOMELARGEFLOATS.xyz + gridcell.zzz * ZINC.xyz ) );
    vec3 highz_mod = vec3( 1.0 / ( SOMELARGEFLOATS.xyz + gridcell_inc1.zzz * ZINC.xyz ) );
    lowz_hash_0 = fract( P * lowz_mod.xxxx );
    highz_hash_0 = fract( P * highz_mod.xxxx );
    lowz_hash_1 = fract( P * lowz_mod.yyyy );
    highz_hash_1 = fract( P * highz_mod.yyyy );
    lowz_hash_2 = fract( P * lowz_mod.zzzz );
    highz_hash_2 = fract( P * highz_mod.zzzz );
}
void FAST32_hash_3D(    vec3 gridcell,
                        out vec4 lowz_hash_0,
                        out vec4 lowz_hash_1,
                        out vec4 lowz_hash_2,
                        out vec4 lowz_hash_3,
                        out vec4 highz_hash_0,
                        out vec4 highz_hash_1,
                        out vec4 highz_hash_2,
                        out vec4 highz_hash_3   )       //  generates 4 random numbers for each of the 8 cell corners
{
    //    gridcell is assumed to be an integer coordinate

    //  TODO:   these constants need tweaked to find the best possible noise.
    //          probably requires some kind of brute force computational searching or something....
    const vec2 OFFSET = vec2( 50.0, 161.0 );
    const float DOMAIN = 69.0;
    const vec4 SOMELARGEFLOATS = vec4( 635.298681, 682.357502, 668.926525, 588.255119 );
    const vec4 ZINC = vec4( 48.500388, 65.294118, 63.934599, 63.279683 );

    //  truncate the domain
    gridcell.xyz = gridcell.xyz - floor(gridcell.xyz * ( 1.0 / DOMAIN )) * DOMAIN;
    vec3 gridcell_inc1 = step( gridcell, vec3( DOMAIN - 1.5 ) ) * ( gridcell + 1.0 );

    //  calculate the noise
    vec4 P = vec4( gridcell.xy, gridcell_inc1.xy ) + OFFSET.xyxy;
    P *= P;
    P = P.xzxz * P.yyww;
    lowz_hash_3.xyzw = vec4( 1.0 / ( SOMELARGEFLOATS.xyzw + gridcell.zzzz * ZINC.xyzw ) );
    highz_hash_3.xyzw = vec4( 1.0 / ( SOMELARGEFLOATS.xyzw + gridcell_inc1.zzzz * ZINC.xyzw ) );
    lowz_hash_0 = fract( P * lowz_hash_3.xxxx );
    highz_hash_0 = fract( P * highz_hash_3.xxxx );
    lowz_hash_1 = fract( P * lowz_hash_3.yyyy );
    highz_hash_1 = fract( P * highz_hash_3.yyyy );
    lowz_hash_2 = fract( P * lowz_hash_3.zzzz );
    highz_hash_2 = fract( P * highz_hash_3.zzzz );
    lowz_hash_3 = fract( P * lowz_hash_3.wwww );
    highz_hash_3 = fract( P * highz_hash_3.wwww );
}

float Interpolation_C2( float x ) { return x * x * x * (x * (x * 6.0 - 15.0) + 10.0); }   //  6x^5-15x^4+10x^3  ( Quintic Curve.  As used by Perlin in Improved Noise.  http://mrl.nyu.edu/~perlin/paper445.pdf )
vec2 Interpolation_C2( vec2 x ) { return x * x * x * (x * (x * 6.0 - 15.0) + 10.0); }
vec3 Interpolation_C2( vec3 x ) { return x * x * x * (x * (x * 6.0 - 15.0) + 10.0); }
vec4 Interpolation_C2( vec4 x ) { return x * x * x * (x * (x * 6.0 - 15.0) + 10.0); }
vec4 Interpolation_C2_InterpAndDeriv( vec2 x ) { return x.xyxy * x.xyxy * ( x.xyxy * ( x.xyxy * ( x.xyxy * vec2( 6.0, 0.0 ).xxyy + vec2( -15.0, 30.0 ).xxyy ) + vec2( 10.0, -60.0 ).xxyy ) + vec2( 0.0, 30.0 ).xxyy ); }
vec3 Interpolation_C2_Deriv( vec3 x ) { return x * x * (x * (x * 30.0 - 60.0) + 30.0); }

float Interpolation_C2_Fast( float x ) { float x3 = x*x*x; return ( 7.0 + ( x3 - 7.0 ) * x ) * x3; }   //  7x^3-7x^4+x^7   ( Faster than Perlin Quintic.  Not quite as good shape. )
vec2 Interpolation_C2_Fast( vec2 x ) { vec2 x3 = x*x*x; return ( 7.0 + ( x3 - 7.0 ) * x ) * x3; }
vec3 Interpolation_C2_Fast( vec3 x ) { vec3 x3 = x*x*x; return ( 7.0 + ( x3 - 7.0 ) * x ) * x3; }
vec4 Interpolation_C2_Fast( vec4 x ) { vec4 x3 = x*x*x; return ( 7.0 + ( x3 - 7.0 ) * x ) * x3; }

float Interpolation_C3( float x ) { float xsq = x*x; float xsqsq = xsq*xsq; return xsqsq * ( 25.0 - 48.0 * x + xsq * ( 25.0 - xsqsq ) ); }   //  25x^4-48x^5+25x^6-x^10     ( C3 Interpolation function.  If anyone ever needs it... :) )
vec2 Interpolation_C3( vec2 x ) { vec2 xsq = x*x; vec2 xsqsq = xsq*xsq; return xsqsq * ( 25.0 - 48.0 * x + xsq * ( 25.0 - xsqsq ) ); }
vec3 Interpolation_C3( vec3 x ) { vec3 xsq = x*x; vec3 xsqsq = xsq*xsq; return xsqsq * ( 25.0 - 48.0 * x + xsq * ( 25.0 - xsqsq ) ); }
vec4 Interpolation_C3( vec4 x ) { vec4 xsq = x*x; vec4 xsqsq = xsq*xsq; return xsqsq * ( 25.0 - 48.0 * x + xsq * ( 25.0 - xsqsq ) ); }


vec4 Perlin3D_Deriv( vec3 P )
{
    //  establish our grid cell and unit position
    vec3 Pi = floor(P);
    vec3 Pf = P - Pi;
    vec3 Pf_min1 = Pf - 1.0;

    //  calculate the hash.
    //  ( various hashing methods listed in order of speed )
    vec4 hashx0, hashy0, hashz0, hashx1, hashy1, hashz1;
    FAST32_hash_3D( Pi, hashx0, hashy0, hashz0, hashx1, hashy1, hashz1 );
    //SGPP_hash_3D( Pi, hashx0, hashy0, hashz0, hashx1, hashy1, hashz1 );

    //  calculate the gradients
    vec4 grad_x0 = hashx0 - 0.49999;
    vec4 grad_y0 = hashy0 - 0.49999;
    vec4 grad_z0 = hashz0 - 0.49999;
    vec4 grad_x1 = hashx1 - 0.49999;
    vec4 grad_y1 = hashy1 - 0.49999;
    vec4 grad_z1 = hashz1 - 0.49999;
    vec4 norm_0 = inversesqrt( grad_x0 * grad_x0 + grad_y0 * grad_y0 + grad_z0 * grad_z0 );
    vec4 norm_1 = inversesqrt( grad_x1 * grad_x1 + grad_y1 * grad_y1 + grad_z1 * grad_z1 );
    grad_x0 *= norm_0;
    grad_y0 *= norm_0;
    grad_z0 *= norm_0;
    grad_x1 *= norm_1;
    grad_y1 *= norm_1;
    grad_z1 *= norm_1;

    //  calculate the dot products
    vec4 dotval_0 = vec2( Pf.x, Pf_min1.x ).xyxy * grad_x0 + vec2( Pf.y, Pf_min1.y ).xxyy * grad_y0 + Pf.zzzz * grad_z0;
    vec4 dotval_1 = vec2( Pf.x, Pf_min1.x ).xyxy * grad_x1 + vec2( Pf.y, Pf_min1.y ).xxyy * grad_y1 + Pf_min1.zzzz * grad_z1;

    //
    //  NOTE:  the following is based off Milo Yips derivation, but modified for parallel execution
    //  http://stackoverflow.com/a/14141774
    //

    //  Convert our data to a more parallel format
    vec4 dotval0_grad0 = vec4( dotval_0.x, grad_x0.x, grad_y0.x, grad_z0.x );
    vec4 dotval1_grad1 = vec4( dotval_0.y, grad_x0.y, grad_y0.y, grad_z0.y );
    vec4 dotval2_grad2 = vec4( dotval_0.z, grad_x0.z, grad_y0.z, grad_z0.z );
    vec4 dotval3_grad3 = vec4( dotval_0.w, grad_x0.w, grad_y0.w, grad_z0.w );
    vec4 dotval4_grad4 = vec4( dotval_1.x, grad_x1.x, grad_y1.x, grad_z1.x );
    vec4 dotval5_grad5 = vec4( dotval_1.y, grad_x1.y, grad_y1.y, grad_z1.y );
    vec4 dotval6_grad6 = vec4( dotval_1.z, grad_x1.z, grad_y1.z, grad_z1.z );
    vec4 dotval7_grad7 = vec4( dotval_1.w, grad_x1.w, grad_y1.w, grad_z1.w );

    //  evaluate common constants
    vec4 k0_gk0 = dotval1_grad1 - dotval0_grad0;
    vec4 k1_gk1 = dotval2_grad2 - dotval0_grad0;
    vec4 k2_gk2 = dotval4_grad4 - dotval0_grad0;
    vec4 k3_gk3 = dotval3_grad3 - dotval2_grad2 - k0_gk0;
    vec4 k4_gk4 = dotval5_grad5 - dotval4_grad4 - k0_gk0;
    vec4 k5_gk5 = dotval6_grad6 - dotval4_grad4 - k1_gk1;
    vec4 k6_gk6 = (dotval7_grad7 - dotval6_grad6) - (dotval5_grad5 - dotval4_grad4) - k3_gk3;

    //  C2 Interpolation
    vec3 blend = Interpolation_C2( Pf );
    vec3 blendDeriv = Interpolation_C2_Deriv( Pf );

    //  calculate final noise + deriv
    float u = blend.x;
    float v = blend.y;
    float w = blend.z;

    vec4 result = dotval0_grad0
        + u * ( k0_gk0 + v * k3_gk3 )
        + v * ( k1_gk1 + w * k5_gk5 )
        + w * ( k2_gk2 + u * ( k4_gk4 + v * k6_gk6 ) );

    result.y += dot( vec4( k0_gk0.x, k3_gk3.x * v, vec2( k4_gk4.x, k6_gk6.x * v ) * w ), vec4( blendDeriv.x ) );
    result.z += dot( vec4( k1_gk1.x, k3_gk3.x * u, vec2( k5_gk5.x, k6_gk6.x * u ) * w ), vec4( blendDeriv.y ) );
    result.w += dot( vec4( k2_gk2.x, k4_gk4.x * u, vec2( k5_gk5.x, k6_gk6.x * u ) * v ), vec4( blendDeriv.z ) );

    //  normalize and return
    result *= 1.1547005383792515290182975610039;        //  (optionally) scale things to a strict -1.0->1.0 range    *= 1.0/sqrt(0.75)
    return result;
}
/// 3d perlin

float rand(vec2 co){
    return fract(sin(dot(co.xy ,vec2(12.9898,78.233))) * 43758.5453);
}

float rampfactor(vec3 p, vec3 o, float sphereradius) {
    float r = (length(p - o) - sphereradius) / 0.5;

    // Compute dat alpha
    float alpha = 1.0;
    if (r > 1.0) {
        alpha = 1.0;
    } else if (r < -1.0) {
        alpha = -1.0;
    } else {
        alpha = (15.0 / 8.0) * r - (10.0/8.0) * r * r * r + (3.0 / 8.0) * r * r * r * r * r;
    }
    return abs(alpha);
}

/////////////////////////////////
// Scene specific stuff
vec4 emit(int scene) {
    bool initnow = bool(init > 0.0);
    vec4 state = texture(inputtex, UV);
    vec4 prev  = texture(inputtex, UV);
    if (scene == 1) {
        if (initnow || (state.w >= 1.0)) {
            state = vec4(-1.0 + 1.1 * cos(UV.x + UV.y + iGlobalTime),
                         0.75 * sin(2324.0 * UV.x + iGlobalTime + 43.0* UV.y + prev.x),
                         0.75 * sin(2.0 * 21394.0 * UV.x + 543.0 * UV.y + 4.0 * iGlobalTime),
                         rand(UV + 4.0 * prev.xy + 5.043*prev.zx + iGlobalTime));
            if (init < 0.5) {
                state.w = 0.0;
                state.x = -1.0;
            }
        }
    }
    else if (scene == 2) {
        if (initnow || (state.w >= 1.0)) {
            vec2 ep = texture(emitpos, UV).xy;
            state = vec4(6.0*0.5 * ep.x,
                         2.0*(ep.y-0.5),
                         0.0,
                         rand(UV + 4.0 * prev.xy + 5.043*prev.zx + iGlobalTime));
        }
    }
    // Static tentacle stuff
    else if (scene == 3) {
        if (initnow || (state.w >= 1.0)) {
            vec3 ep = texture(emitpos, UV).xyz;
            state = vec4(0.125* ep.x + 0.05*rand(ep.xy + UV + iGlobalTime),
                         0.125* ep.y + 0.05*rand(ep.xy + UV.yx + iGlobalTime),
                         0.125* ep.z + 0.05*rand(ep.yx + UV + iGlobalTime), //0.25* ep.z,
                         rand(UV + iGlobalTime));
        }
    }
    // epoch logo
    else if (scene == 5) {
        if (initnow || (state.w >= 1.0)) {
            vec3 ep = texture(emitpos, UV).xyz;
            state = vec4(16.0*0.125* ep.x + 0.1*rand(ep.xy + UV + iGlobalTime),
                         9.0*0.125* ep.y + 0.1*rand(ep.zy + -2.4*UV + 5.4*iGlobalTime),
                         0.125* ep.z, //0.25* ep.z,
                         rand(UV + iGlobalTime));
        }
    }
    // alkujuttu
    else if (scene == 6) {
        if (initnow || (state.w >= 1.0)) {
            state = vec4(-1.0 + 0.125*1.1 * (rand(UV.xy + iGlobalTime) - 0.5),
                         0.75 * 0.125*sin(2324.0 * UV.x + iGlobalTime + 43.0* UV.y + prev.x),
                         0.75 * sin(2.0 * 21394.0 * UV.x + 543.0 * UV.y + 4.0 * iGlobalTime),
                         rand(UV + 4.0 * prev.xy + 5.043*prev.zx + iGlobalTime));
            if (init < 0.5) {
                state.w = 0.0;
                state.x = -1.0;
            }
        }
    }
    // taustahomma
    else if (scene == 7) {
        if (initnow || (state.w >= 1.0)) {
            state = 0.0*vec4(1.1 * (rand(UV.xy + iGlobalTime) - 0.5),
                         0.75 * 0.125*sin(2324.0 * UV.x + iGlobalTime + 43.0* UV.y + prev.x),
                         0.75 * sin(2.0 * 21394.0 * UV.x + 543.0 * UV.y + 4.0 * iGlobalTime),
                         rand(UV + 4.0 * prev.xy + 5.043*prev.zx + iGlobalTime));
        }
    }

    // ...
    else if (!initnow) {
        return texture(inputtex, UV);
    }

    return state;
}



////////////////////////////////////
float ccoef(vec4 state, int scene, inout vec3 nhat) {
    if ((scene == 1) || (scene == 2)) {
        const vec3 o = vec3(2.4, 0.0, 0.0);
        const float sphereradius = 0.5;
        float a = rampfactor(state.xyz, o, sphereradius);
        nhat = normalize(state.xyz - o);
    }

    return 1.0;
}

vec4 flow(int scene) {
    if ((scene == 1) || (scene == 2)) {
        return vec4(0.0, 0.0, 4.0*0.8, 0.0);
    }
    return vec4(0.0);
}
/////////////////////////////////////

vec4 simulate(int scene, vec4 state, float dt) {
    // Generate perlin noise potential field and compute the derivatives of the noise field.
    vec4 perlinshitA = Perlin3D_Deriv(4.0 * state.xyz + vec3(state.xy, 0.25*iGlobalTime));
    vec4 perlinshitB = Perlin3D_Deriv(4.0 * state.xyz + vec3(state.xy + vec2(4.02324, -10.2498), 25.02 + 0.25*iGlobalTime));
    vec4 perlinshitC = Perlin3D_Deriv(4.0 * state.xyz + vec3(state.xy - vec2(-3.02324, -10.2498), -12.02 + 0.25*iGlobalTime))
                     + flow(scene);
    
    // Ramp stuff
    vec3 nhat;
    float a = ccoef(state, scene, nhat);
//    vec3 nhat = normalize(state.xyz - o);
    perlinshitA.yzw = a * perlinshitA.yzw + (1.0 - a) * vec3(dot(nhat, vec3(perlinshitA.y, perlinshitB.y, perlinshitC.y)),
                                                             dot(nhat, vec3(perlinshitA.z, perlinshitB.z, perlinshitC.z)),
                                                             dot(nhat, vec3(perlinshitA.w, perlinshitB.w, perlinshitC.w))) * nhat.x;
    perlinshitB.yzw = a * perlinshitB.yzw + (1.0 - a) * vec3(dot(nhat, vec3(perlinshitA.y, perlinshitB.y, perlinshitC.y)),
                                                             dot(nhat, vec3(perlinshitA.z, perlinshitB.z, perlinshitC.z)),
                                                             dot(nhat, vec3(perlinshitA.w, perlinshitB.w, perlinshitC.w))) * nhat.y;
    perlinshitC.yzw = a * perlinshitC.yzw + (1.0 - a) * vec3(dot(nhat, vec3(perlinshitA.y, perlinshitB.y, perlinshitC.y)),
                                                             dot(nhat, vec3(perlinshitA.z, perlinshitB.z, perlinshitC.z)),
                                                             dot(nhat, vec3(perlinshitA.w, perlinshitB.w, perlinshitC.w))) * nhat.z;

    // Compute the velocity etc
    vec4 velocity = vec4(perlinshitC.z - perlinshitB.w,
                         perlinshitA.w - perlinshitC.y,
                         perlinshitB.y - perlinshitA.z,
                         0.5);
    if (scene != 7) {
        return state + dt * vec4(veloscale * velocity.xyz, velocity.w) * deltat / 16.67;
    } else {
        return state + dt * vec4(veloscale * velocity.xyz, 0.0) * deltat / 16.67;
    }
}

/////////////////////////////////

void main() {
    int scenein = int(scenenum);
    // Initialize particle position if needed.
    vec4 state = emit(scenein);

    // Simulation
    color = simulate(scenein, state, 0.001);
}
